/**********************************************************
 *	tables.js - Author: Samnan ur Rehman                   *
 *	This file is a part of MyWebSQL package                *
 * collection of functions to help with                   *
 * tables sorting/editing etc                             *
 **********************************************************/

var curEditField = null;         // current edited field
var curEditFieldId = null;       // id of the parent <tr> of edited field
var editData = new Array();      // whole of current editing data until saved or discarded
var fieldInfo = null;            // information about fields in record set
var editOptions = null;
var editToolbarTimer = null;     // timer for hiding editing toolbar during field editing

// options can include
// highlight: boolean: highlights row on mouse over
// selectable: boolean: makes row selectable
// editable: boolean: makes grid editable
// sortable: boolean: makes table sorting possible using its header

function setupTable(id, opt)
{
	opt.editEvent ? void(0) : opt.editEvent = 'dblclick';
	opt.editFunc ? void(0) : opt.editFunc = editTableCell;

	if (opt.sortable)
	{
		sorttable.DATE_RE = /^(\d\d?)[\/\.-](\d\d?)[\/\.-]((\d\d)?\d\d)$/;
		table = document.getElementById(id);
		sorttable.makeSortable(table);
	}

	if (opt.highlight)
	{
		$('#'+id+' tr').hover(function()
		{
			$(this).addClass("rowHL");
		}, function(){
			$(this).removeClass('rowHL');
		});
	}

	if (opt.selectable)
	{
		$('#'+id+' tr').bind('click', function()
		{
			if (selectedRow != null)
				$(selectedRow).removeClass('rowSel');
			$(this).addClass("rowSel");
			selectedRow = this;
		});
	}

	editOptions = null;
	if (opt.editable)
	{
		editOptions = opt;
		$('#'+id+' td.edit').bind(opt.editEvent, opt.editFunc);
	}
}

function editTableCell()
{
	td = $(this);
	if (curEditField != null)
		closeEditor(true);

	txt = td.text();
	tstyle = td.hasClass('tr') ? "right" : "left";

	td.data('defText', txt);

	curEditField = this;
	curEditFieldId = this.parentNode.id.substr(2);
	fid = str_replace("r" + curEditFieldId + "f", "", curEditField.id);
	fi = getFieldInfo(fid);

	p = Position.get(this);
	w = td.width();
	h = td.height();

	keyEvent = browser.isFirefox ? 'keypress' : 'keydown';
	td.html( '<form name="cell_editor_form" class="cell_editor_form" action="javascript:void(0);">'+
		'<input type="text" name="cell_editor" class="cell_editor"' +
		' style="text-align:' + tstyle + ';width: ' + w + 'px;" />' + '</form>');
	input = td.find('input');
	input.val(txt).bind(keyEvent, checkEditField ).blur( function() {closeEditor(true)} ).focus();

	if (document.getElementById('editToolbar'))
	{
		leftPos = p.left;
		if (leftPos + $('#editToolbar').width() >= document.body.clientWidth)
			leftPos = document.body.clientWidth - $('#editToolbar').width() - 20;   // keep some space off as there maybe scrollbars
		$('#editToolbar').css({"display":"block", "left":leftPos + "px","top":(p.top-p.height-2) + "px"});
	}
}

function closeEditor(upd, value)
{
	txt = '';
	if (upd)
	{
		var xt = new Object();
		if (arguments.length > 1 && value == null)
		{
			xt.value = "NULL";
			xt.setNull = true;
		}
		else
		{
			txt = xt.value = $(curEditField).find('input').val();
			xt.setNull = false;
		}

		// if not modified, don't bother
		if ( (xt.value != $(curEditField).data('defText')) || (xt.setNull && !$(curEditField).hasClass("tnl")) || (!xt.setNull && $(curEditField).hasClass('tnl')) )
		{
			if (typeof editData[curEditFieldId] != "object")
			{
				editData[curEditFieldId] = new Array();
			}
			fid = str_replace("r" + curEditFieldId + "f", "", curEditField.id);
			editData[curEditFieldId][fid] = xt;

			if (typeof showNavBtn == "function")
				showNavBtn('update');

			if(xt.setNull)
				$(curEditField).removeClass('tl').addClass('tnl');
			else
				$(curEditField).removeClass('tnl').addClass('tl');

			txt = xt.value;
			//txt = str_replace("<", "&lt;", xt.value);
			//txt = str_replace(">", "&gt;", txt);
		}
	}
	else
		txt = $(curEditField).data('defText');

	$(curEditField).text(txt);
	curEditField = null;

	if (document.getElementById('editToolbar'))
		editToolbarTimer = window.setTimeout(function() { document.getElementById('editToolbar').style.display = "none"; editToolbarTimer=null; }, 100 );
}

function checkEditField(event)
{
	if (!event)
		event = window.event;

	if (event.keyCode == 13 || event.keyCode == 9)
	{
		elem = false;
		if (event.keyCode == 9)
		{
			row = parseInt( curEditField.parentNode.id.substr(2) );
			col = parseInt( str_replace("r" + curEditFieldId + "f", "", curEditField.id) );
			elem = event.shiftKey ? $(curEditField).prev('.edit') : $(curEditField).next('.edit');
		}
		closeEditor(true);
		if (elem && elem.length)    // edit next or previous element
			elem.trigger(editOptions.editEvent);
	}
	else if (event.keyCode == 27)
		closeEditor(false);
	else if (event.keyCode == KEYCODE_SETNULL && event.shiftKey == 1)
		closeEditor(true, null);
}

/* used while creating or editing tables in the create table module */
function editTableField()
{
	td = $(this);
	if (curEditField != null)
		closeEditor(true);

	txt = td.text();
	tstyle = td.hasClass('tr') ? "right" : "left";

	td.data('defText', txt);

	curEditField = this;
	curEditFieldId = this.parentNode.id.substr(2);
	fid = str_replace("r" + curEditFieldId + "f", "", curEditField.id);
	fi = getFieldInfo(fid);

	p = Position.get(this);
	w = td.width();
    h = td.height();

	td.html( '<form name="cell_editor_form" action="javascript:void(0);">'+
		'<input type="text" name="cell_editor" class="cell_editor"' +
		' style="text-align:' + tstyle + ';width: ' + w + 'px;" />' + '</form>');
	input = td.find('input');
	input.val(txt).keypress( checkEditField ).blur( function() {closeEditor(true)} ).focus();

	if (document.getElementById('editToolbar'))
	{
		leftPos = p.left;
		if (leftPos + $('#editToolbar').width() >= document.body.clientWidth)
			leftPos = document.body.clientWidth - $('#editToolbar').width() - 20;   // keep some space off as there maybe scrollbars
		$('#editToolbar').css({"display":"block", "left":leftPos + "px","top":(p.top-p.height-2) + "px"});
	}
}
