<?php

	/********************************************************************
	*	maillib.php - Authors:											*
	*		Samnan ur Rehman, Zeeshan Khan, Faraz Ahmed Ikram,			*
	*		Danish Zahur												*
	*	Contains modified code from CMailFile and smtp class 			*
	*	Taken from phpclasses.net long time ago (author names required)	*
	*	This file is a part of MyWebSQL package							*
	*	PHP5 compatible													*
	*********************************************************************/

if (!defined("MAILLIB_INCLUDED"))
{
	define("MAILLIB_INCLUDED", "1");

	class CMailFile {
		var $subject;
		var $addr_to;
		var $text_body;
		var $content_type;
		var $text_encoded;
		var $mime_headers;
		var $messagebody;
		var $attached_files = 0;
		var $mime_boundary = "0-397292483-3920387554=:22729";
		var $smtp_headers;
		var $reply_to;
		var $char_encoding;
		var $attached_inline_files;
		var $TIMEZONES;
		
		// default variables left so the object can be made and its mail is created later on
		function CMailFile($subject="",$to="",$from="",$msg="",$filename="",$mimetype = "application/octet-stream", $mime_filename = false)
		{
			$this->subject = $subject;
			$this->addr_to = $to;
			$this->messagebody = $msg;
			$this->reply_to  = $from;
			$this->content_type = "text/plain";
			if ($filename != "")
				$this->text_encoded = $this->attach_file($filename,$mimetype,$mime_filename);
			$this->char_encoding = "windows-1252";

			$this->attached_inline_files = 0;
			
			$this->TIMEZONES = array();
			$this->TIMEZONES["GMT"]  = "+0000";
			$this->TIMEZONES["UTC"]  = "+0000";
			$this->TIMEZONES["BST"]  = "+0100";
			$this->TIMEZONES["IST"]  = "+0100";
			$this->TIMEZONES["WET"]  = "+0000";
			$this->TIMEZONES["WEST"]  = "+0100";
			$this->TIMEZONES["CET"]  = "+0100";
			$this->TIMEZONES["CEST"]  = "+0200";
			$this->TIMEZONES["EET"]  = "+0200";
			$this->TIMEZONES["EEST"]  = "+0300";
			$this->TIMEZONES["PKT"]  = "+0500";
		}

		function setCharEncoding($encoding)			//set the given encoding type
		{
			$this->char_encoding = $encoding;	
		}
		
		function write_body($msgtext)
		{
			if ($this->attached_files > 0)
			{
				$out = "--" . $this->mime_boundary . "\n";
				$out = $out . "Content-Type: $this->content_type; charset=\"".$this->char_encoding."\";\n";
				$out = $out . "Content-Disposition: inline\n\n";
			}
			$out = $out . $msgtext . "\n";
			return $out;
		}

		function write_mimeheaders()
		{
			$out = "MIME-Version: 1.0\r\n";		// DO NOT REMOVE \r\n. Exim breaks apart mail if not done so
			if ($this->attached_inline_files > 0)
				$out .= "Content-Type: multipart/related; ";
			else if ($this->attached_files > 0)
				$out = $out . "Content-Type: multipart/mixed; ";
			else
				$out = $out . "Content-Type: $this->content_type; charset=\"".$this->char_encoding."\";\n";	///added 12-14-2005 to set char encoding for non multipart
			$out = $out . "boundary=\"$this->mime_boundary\"\n";
			return $out;
		}

		function write_smtpheaders($addr_from)
		{
			$out = "From: $addr_from\n";
			return $out;
		}

		function attach_file($filename,$mimetype,$mime_filename)
		{
			$encoded = $this->encode_file($filename);
			if ($mime_filename)
				$filename = $mime_filename;
			
			$out = "--" . $this->mime_boundary . "\n";
			$out = $out . "Content-Type: " . $mimetype . "; name=\"$filename\";\n";				
			$out = $out . "Content-Transfer-Encoding: base64\n";
			$out = $out . "Content-Description: $filename\n";
			$out = $out . "Content-Disposition: attachment; filename=\"$filename\"\n\n";
			$out = $out . $encoded . "\n";
			$this->attached_files++;
			return $out; 
		}

		// $mimetype according to file .i.e. for a jpg file $mimetype should be 'image/jpg'
		function attach_inline($filename, $replacement="", $mimetype="image/jpg")
		{
			$encoded = $this->encode_file($filename);
			
			if ($encoded)
			{
				debugLog("file encoded successfully");
				$filename = basename($filename);
				$randNumber = rand(0, 10000);
				$contentID = "$filename@". substr(md5("$filename$randNumber"), 0, 10);
				$this->content_type = "text/html";	//	To show image inline
				if ($replacement)
					$this->messagebody = str_replace($replacement, $contentID, $this->messagebody);
				
				$out = "--" . $this->mime_boundary . "\n";
				$out .= "Content-Type: " . $mimetype . "; name=\"$filename\";\n";		
				$out .= "Content-Transfer-Encoding: base64\n";
				$out .= "Content-Description: $filename\n";
				$out .= "Content-ID: <$contentID>\n";
				$out .= "Content-Disposition: attachment; filename=\"$filename\"\n\n";
				$out .= $encoded . "\n";
				
				$this->attached_inline_files++;
				$this->attached_files++;
				$this->text_encoded .= $out;
			}
		}
		
		function attach_other($filename,$mime_filename,$mimetype="application/octet-stream")
		{
			$encoded = $this->encode_file($filename);
			if ($mime_filename)
				$filename = $mime_filename;
			
			$out = "--" . $this->mime_boundary . "\n";
			$out = $out . "Content-Type: " . $mimetype . "; name=\"$filename\";\n";		
			$out = $out . "Content-Transfer-Encoding: base64\n";
			$out = $out . "Content-Description: $filename\n";
			$out = $out . "Content-Disposition: attachment; filename=\"$filename\"\n\n";
			$out = $out . $encoded . "\n";
			
			$this->attached_files++;
			$this->text_encoded .= $out;
		}
		
		function attach_data($data, $mime_filename ,$mimetype = "application/octet-stream")
		{
			$encoded = $this->my_chunk_split(base64_encode($data));
			
			$out = "--" . $this->mime_boundary . "\n";
			$out = $out . "Content-Type: " . $mimetype . "; name=\"$mime_filename\";\n";		
			$out = $out . "Content-Transfer-Encoding: base64\n";
			$out = $out . "Content-Description: $mime_filename\n";
			$out = $out . "Content-Disposition: attachment; filename=\"$mime_filename\"\n\n";
			$out = $out . $encoded . "\n";
			
			$this->attached_files++;
			$this->text_encoded .= $out;
		}
		
		function sendfile() // on function used to generate data and send it.
		{
			$sendData = $this->generateMail();
			$x = $this->sendMail($sendData);
			return $x;
		}

		function getrawmail()
		{
			return ($this->messagebody);
		}
		
		function generateMail() // this function returns the mail with ALL headers and stuff. that way all that is left to do is send that mail.
		{
			$this->text_body = $this->write_body($this->messagebody);
			$this->mime_headers = $this->write_mimeheaders();
			$headers = $this->smtp_headers . $this->mime_headers;
			$message = $this->text_body.$this->text_encoded;
			
			$temp = array();
			array_push($temp, $this->addr_to);
			$send_params['recipients']	= $temp;
			unset($temp);
			$temp = array();
			array_push($temp, "From: $this->reply_to");
			array_push($temp, "To: $this->addr_to");
			array_push($temp, "Subject: $this->subject");
			//*********************** TIME STAMP ************************//
			$timestamp = date("D, j M Y G:i:s") . " " . $this->TIMEZONES[trim(date("T"))];
			debugLog("time stamp ->$timestamp ");
			array_push($temp, "Date: $timestamp");
			array_push($temp, "header: $headers");
			
			$send_params['headers']		= $temp;
			$send_params['from']		= $this->reply_to;
			$send_params['body']		= $message;	
			
			return $send_params;		
		}
		
		function sendMail($mail) // function simply mails the data. the data must have all the headers and stuff
		{
			if(USE_SENDMAIL)	// If USE_SENDMAIL is 1 then use built-in mail function
				return $this->send($mail);
			
			$smtp = smtp::connect();
			if(!is_object($smtp))
				return false;
			else if(!$smtp->send($mail))
			{
				foreach($smtp->errors as $temp)
				{
					debugLog("smtp: $temp");
				
				}return false;	
			}
			return TRUE;
		}
		
		function send($mail)	// function send mail using php built-in mail function.
		{
			if(!defined('CRLF'))
				define('CRLF', "\r\n", TRUE);
			
			$headers = str_replace(CRLF.'.', CRLF.'..', trim(implode(CRLF, $mail['headers'])));
			$body    = str_replace(CRLF.'.', CRLF.'..', $mail['body']);
			$body    = $body[0] == '.' ? '.'.$body : $body;
			
			if(mail($this->addr_to, $this->subject, $body, $headers))
				return TRUE;
			else
				return FALSE;
		}
		
		function encode_file($sourcefile)
		{
			if (is_readable($sourcefile))
			{
				$fd = fopen($sourcefile, "r");
				$contents = fread($fd, filesize($sourcefile));
				$encoded = $this->my_chunk_split(base64_encode($contents));
				fclose($fd);	
			}
			return $encoded;
		}
		function my_chunk_split($str)
		{
			$out = chunk_split($str, 76, "\r\n");
			return $out;
		}
	}

	define('SMTP_STATUS_NOT_CONNECTED', 1, TRUE);
	define('SMTP_STATUS_CONNECTED', 2, TRUE);

	class smtp{

		var $authenticated;
		var $connection;
		var $recipients;
		var $headers;
		var $timeout;
		var $errors;
		var $status;
		var $body;
		var $from;
		var $host;
		var $port;
		var $helo;
		var $auth;
		var $user;
		var $pass;

		/***************************************
		** Constructor function. Arguments:
		** $params - An assoc array of parameters:
		**
		**   host    - The hostname of the smtp server		Default: localhost
		**   port    - The port the smtp server runs on		Default: 25
		**   helo    - What to send as the HELO command		Default: localhost
		**             (typically the hostname of the
		**             machine this script runs on)
		**   auth    - Whether to use basic authentication	Default: FALSE
		**   user    - Username for authentication			Default: <blank>
		**   pass    - Password for authentication			Default: <blank>
		**   timeout - The timeout in seconds for the call	Default: 5
		**             to fsockopen()
		***************************************/

		function smtp($params = array()){
			
			if(!defined('CRLF'))
				define('CRLF', "\r\n", TRUE);

			$this->authenticated	= FALSE;			
			$this->timeout			= 5;
			$this->status			= SMTP_STATUS_NOT_CONNECTED;
			if (defined("SMTP_SERVER"))
				$this->host				= SMTP_SERVER;
			else
				$this->host				= 'localhost';
			
			$this->port				= 25;
			$this->helo				= 'localhost';
			$this->auth				= TRUE;
			$this->user				= SMTP_USERID;
			$this->pass				= SMTP_PASSWORD;
			$this->errors   		= array();

			foreach($params as $key => $value)
				$this->$key = $value;
		}

		/***************************************
		** Connect function. This will, when called
		** statically, create a new smtp object, 
		** call the connect function (ie this function)
		** and return it. When not called statically,
		** it will connect to the server and send
		** the HELO command.
		***************************************/

		function &connect($params = array())
		{
			if(!isset($this->status)){
				$obj = new smtp($params);
				if($obj->connect())
					$obj->status = SMTP_STATUS_CONNECTED;
				return $obj;

			}else{
				$this->connection = @fsockopen($this->host, $this->port, $errno, $errstr, $this->timeout);
				if(!$this->connection)
					return false;
				if(function_exists('socket_set_timeout'))
					@socket_set_timeout($this->connection, 5, 0);
				$greeting = $this->get_data();
				if(is_resource($this->connection)){
					return $this->auth ? $this->ehlo() : $this->helo();
				}else{
					$this->errors[] = 'Failed to connect to server: '.$errstr;
					return FALSE;
				}
			}
		}

		/***************************************
		** Function which handles sending the mail.
		** Arguments:
		** $params	- Optional assoc array of parameters.
		**            Can contain:
		**              recipients - Indexed array of recipients
		**              from       - The from address. (used in MAIL FROM:),
		**                           this will be the return path
		**              headers    - Indexed array of headers, one header per array entry
		**              body       - The body of the email
		**            It can also contain any of the parameters from the connect()
		**            function
		***************************************/

		function send($params = array()){
		
			foreach($params as $key => $value)			
				$this->set($key, $value);

			if($this->is_connected()){
				// Do we auth or not? Note the distinction between the auth variable and auth() function
				if($this->auth AND !$this->authenticated)
					if(!$this->auth())
						return FALSE;
				
				$this->mail($this->from);
				if(is_array($this->recipients))
					foreach($this->recipients as $value)
						$this->rcpt($value);
				else
					$this->rcpt($this->recipients);
				
				if(!$this->data())
					return FALSE;
				// Transparency
				$headers = str_replace(CRLF.'.', CRLF.'..', trim(implode(CRLF, $this->headers)));
				$body    = str_replace(CRLF.'.', CRLF.'..', $this->body);
				$body    = $body[0] == '.' ? '.'.$body : $body;
				
				$this->send_data($headers);
				$this->send_data('');
				$this->send_data($body);
				$this->send_data('.');
				
				$result = (substr(trim($this->get_data()), 0, 3) === '250');
				//$this->rset();
				return $result;
			}else{
				$this->errors[] = 'Not connected!';
				return FALSE;
			}
		}
		
		/***************************************
		** Function to implement HELO cmd
		***************************************/

		function helo(){
			if(is_resource($this->connection)
					AND $this->send_data('HELO '.$this->helo)
					AND substr(trim($error = $this->get_data()), 0, 3) === '250' ){

				return TRUE;

			}else{
				$this->errors[] = 'HELO command failed, output: ' . trim(substr(trim($error),3));
				return FALSE;
			}
		}
		
		/***************************************
		** Function to implement EHLO cmd
		***************************************/

		function ehlo(){
			if(is_resource($this->connection)
					AND $this->send_data('EHLO '.$this->helo)
					AND substr(trim($error = $this->get_data()), 0, 3) === '250' ){

				return TRUE;

			}else{
				$this->errors[] = 'EHLO command failed, output: ' . trim(substr(trim($error),3));
				return FALSE;
			}
		}
		
		/***************************************
		** Function to implement RSET cmd
		***************************************/

		function rset(){
			if(is_resource($this->connection)
					AND $this->send_data('RSET')
					AND substr(trim($error = $this->get_data()), 0, 3) === '250' ){

				return TRUE;

			}else{
				$this->errors[] = 'RSET command failed, output: ' . trim(substr(trim($error),3));
				return FALSE;
			}
		}
		
		/***************************************
		** Function to implement QUIT cmd
		***************************************/

		function quit(){
			if(is_resource($this->connection)
					AND $this->send_data('QUIT')
					AND substr(trim($error = $this->get_data()), 0, 3) === '221' ){

				fclose($this->connection);
				$this->status = SMTP_STATUS_NOT_CONNECTED;
				return TRUE;

			}else{
				$this->errors[] = 'QUIT command failed, output: ' . trim(substr(trim($error),3));
				return FALSE;
			}
		}
		
		/***************************************
		** Function to implement AUTH cmd
		***************************************/

		function auth(){
			if(is_resource($this->connection)
					AND $this->send_data('AUTH LOGIN')
					AND substr(trim($error = $this->get_data()), 0, 3) === '334'
					AND $this->send_data(base64_encode($this->user))			// Send username
					AND substr(trim($error = $this->get_data()),0,3) === '334'
					AND $this->send_data(base64_encode($this->pass))			// Send password
					AND substr(trim($error = $this->get_data()),0,3) === '235' ){

				$this->authenticated = TRUE;
				return TRUE;

			}else{
				$this->errors[] = 'AUTH command failed: ' . trim(substr(trim($error),3));
				return FALSE;
			}
		}

		/***************************************
		** Function that handles the MAIL FROM: cmd
		***************************************/
		
		function mail($from)
		{
			if($this->is_connected()
				AND $this->send_data('MAIL FROM:<'.$from.'>')
				AND substr(trim($this->get_data()), 0, 2) === '250' )
					return TRUE;
			else
				return FALSE;
			
		}

		/***************************************
		** Function that handles the RCPT TO: cmd
		***************************************/
		
		function rcpt($to)
		{
			if($this->is_connected()
				AND $this->send_data('RCPT TO:<'.$to.'>')
				AND substr(trim($error = $this->get_data()), 0, 2) === '25' )
					return TRUE;
			else{				
				$this->errors[] = trim(substr(trim($error), 3));
				return FALSE;
			}
		}

		/***************************************
		** Function that sends the DATA cmd
		***************************************/

		function data()
		{
			if($this->is_connected()
				AND $this->send_data('DATA')
				AND substr(trim($error = $this->get_data()), 0, 3) === '354' ){
					return TRUE;

			}else{
				$this->errors[] = trim(substr(trim($error), 3));
				return FALSE;
			}
		}

		/***************************************
		** Function to determine if this object
		** is connected to the server or not.
		***************************************/

		function is_connected()
		{
			return (is_resource($this->connection) AND ($this->status === SMTP_STATUS_CONNECTED));
		}

		/***************************************
		** Function to send a bit of data
		***************************************/

		function send_data($data)
		{
			if(is_resource($this->connection))
				return (fwrite($this->connection, $data.CRLF, strlen($data)+2));
			else
				return FALSE;
		}

		/***************************************
		** Function to get data.
		***************************************/

		function &get_data(){

			$return = '';
			$line   = '';
			$loops  = 0;

			if(is_resource($this->connection)){
				while((strpos($return, CRLF) === FALSE OR substr($line,3,1) !== ' ') AND $loops < 100){
					$line    = fgets($this->connection, 512);
					$return .= $line;
					$loops++;
				}
				return $return;
			}else
				return FALSE;
		}

		/***************************************
		** Sets a variable
		***************************************/
		
		function set($var, $value){
			$this->$var = $value;
			return TRUE;
		}
	}
	
	function is_valid_email($address)
	{
		return ( ! preg_match("/^([a-z0-9\+_\-]+)(\.[a-z0-9\+_\-]+)*@([a-z0-9\-]+\.)+[a-z]{2,6}$/ix", $address)) ? FALSE : TRUE;
	}
}
?>