<?php

	/********************************************************************
	*	util.php - Author: Samnan ur Rehman								*
	*	This file is a part of MyWebSQL package							*
	*	Core functions for server side mysql related functionality		*
	*	PHP5 compatible													*
	*********************************************************************/

	include('functions.php');

	function showDBError()
	{
		return "<div id=\"srcmsg\" style=\"text-align:center\">Database connection failed to the server.<br/>Host: ".DB_HOST.", User: ".DB_USER."<br /></div>";
	}

	function getDbName()
	{
		return v($_SESSION["db"]);
	}

	function printDbList(&$db, &$HTML)
	{
		global $DB_CONFIG;
		$dblist = v($_SESSION["dblist"]) ? array(v($_SESSION["db"])) : $db->getDatabases();

		// if there is one database for the given userid, lets select it to cut out one extra step
		if (! v($_SESSION["db"]) )
		{
			$stDb = 0; $selDb = "";
			foreach($dblist as $dbname)
			{
				if ($dbname == "information_schema" || $dbname == "mysql" || $dbname == "test")
					$stDb++;
				else
					$selDb = $dbname;
			}
			if (count($dblist) == ($stDb+1))
			{
				$_SESSION["db"] = $selDb;
				$db->selectDb($_SESSION["db"]);
			}
		}
		// --- end of automatic selection logic

		if (getDbName())
		{
			$HTML->select("dblist", "id='dblist' onchange='selectDatabase()'", "", "width:100%");
			foreach($dblist as $dbname)
				$HTML->option($dbname, $dbname, v($_SESSION["db"]) == $dbname ? "selected=\"selected\"" : "");
			$HTML->endselect();
		}
		else
		{
			print '<div style="padding:3px;font-weight:bold">Select a database to begin.</div>';
		}
		return $dblist;
	}

	function getDBClass()
	{
		// use least problematic solution when session isn't ready yet
		if (v($_SESSION['init']) !== 1)
			$lib = "lib/db4.php";
		else if (v($_SESSION["dbver"]) && ($_SESSION["dbver"] < 5 ) )
			$lib = "lib/db4.php";
		else if (defined("USE_MYSQLI") && USE_MYSQLI == 1 && extension_loaded('mysqli') )
			$lib = "lib/dbex.php";
		else
			$lib = "lib/db.php";

		return $lib;
	}

	function createBasicInfoPage(&$DB, &$HTML)
	{
		$ver = $ver2 = $cs1 = $cs2 = $cs3 = $cs4 = "";
		if ($DB->query("show variables"))
		{
			while($row = $DB->fetchRow())
			{
				switch($row[0])
				{
					case "version": $ver = $row[1]; break;
					case "version_comment": $ver2 = $row[1]; break;
					case "character_set_server": $cs1 = $row[1]; break;
					case "character_set_client": $cs2 = $row[1]; break;
					case "character_set_database": $cs3 = $row[1]; break;
					case "character_set_results": $cs4 = $row[1]; break;
				}
			}

			print "<div class='heading1'>Server information</div>\n";
			print '<div class="hinfo">';
				print "<div class=\"label\">Mysql version</div><div class=\"info\">$ver</div>\n";
				print "<div class=\"label\">Version comment</div><div class=\"info\">$ver2</div>\n";
			print "</div>";
			print "<div class='heading1'>Character sets</div>\n";
			print '<div class="hinfo">';
				print "<div class=\"label\">Server character set</div><div class=\"info\">$cs1</div>\n";
				print "<div class=\"label\">Client character set</div><div class=\"info\">$cs2</div>\n";
			print "</div>";
			print '<div class="hinfo">';
				print "<div class=\"label\">Database character set</div><div class=\"info\">$cs3</div>\n";
				print "<div class=\"label\">Results character set</div><div class=\"info\">$cs4</div>\n";
			print "</div>";
			print "<div class='heading1'>Browser information</div>\n";
			print '<div class="hinfo">';
				print "<div class=\"label\">Browser</div><div class=\"info\" id=\"divBr1\"></div>\n";
				print "<div class=\"label\">Version</div><div class=\"info\" id=\"divBr2\"></div>\n";
			print "</div>";
		}
	}

	function doWork(&$db)
	{
		//traceMessage("doWork ... [".v($_REQUEST[type])."][".v($_REQUEST[id])."][".v($_REQUEST[name])."]");
		// contents of the iframe
		startForm();

        if ( isset($_REQUEST["type"]) )
        {
			$_REQUEST["query"] = trim(v($_REQUEST["query"], ""), " \t\r\n;");
       		$module = "modules/".$_REQUEST["type"].".php";
            if (ctype_alpha($_REQUEST["type"]) && file_exists($module))
            {
                include($module);
                processRequest($db);
            }
            else
                createErrorPage();		// unidentified type requested
		}
		print "</form>\n";
		print "</body></html>";
	}

	function createResultGrid(&$db)
	{
		traceMessage("createResultGrid...");
		unset($_SESSION["res"]);
		unset($_SESSION["primary_key"]);
		unset($_SESSION["unique_key"]);
		unset($_SESSION["multiple_key"]);
		unset($_SESSION["table"]);
		$_SESSION["res"] = array();		// result blob data

		// <form element is moved to js, see comments there
		print "<div name='results' id='results'>";
		print "<table cellspacing=\"0\" width='100%' border=\"0\" class='results' id=\"dataTable\"><thead>\n";

		$f = $db->getFieldInfo();
		// see if all fields come from one table so we can allow editing
		$_SESSION["table"] = $f[0]->table;
		for($i=1; $i<count($f);$i++)
		{
			if ($f[$i]->table != $_SESSION["table"])	// bail out, more than one table data
			{
				$_SESSION["table"] = "";
				break;
			}
		}

		// desc command returns table name as COLUMNS, so we make it empty
		if ($_SESSION["table"] == "COLUMNS")
			$_SESSION["table"] = "";

		$ed = ($_SESSION["table"] == "" ? false : true) && ($db->numRows() > 0);
		// ------------ print header -----------
		print "<tr id=\"fhead\">";
		print "<th class=\"th\">#</th>";

		if ($ed)
			print "<th class=\"th_nosort\"><input name=\"ckrAll\" type=\"checkbox\" onclick=\"resultSelectAll()\" title=\"Select/unselect All records\" /></th>";

		$v = "";
		$_SESSION["primary_key"] = '';
		$_SESSION["unique_key"] = '';
		$_SESSION["multiple_key"] = '';
		$fieldNames = '';
		//error_log(print_r($f, 1));
		$fieldInfo = json_encode($f);
		foreach($f as $fn)
		{
			$cls = $fn->type == 'numeric' ? "th_numeric" : "th";
			print "<th nowrap=\"nowrap\" class='$cls'>";
			if ($fn->pkey == 1)
			{
				$_SESSION["primary_key"] .= ",". $fn->name;
				print "<img align=\"absmiddle\" src=\"themes/".THEME_PATH."/images/pk.png\" title=\"Primary key\" alt=\"\" />&nbsp;";
			}
			else if ($fn->ukey == 1)
			{
				$_SESSION["unique_key"] .= ",". $fn->name;
				print "<img align=\"absmiddle\" src=\"themes/".THEME_PATH."/images/uk.png\" title=\"Unique key\" />&nbsp;";
			}
			else if ($fn->mkey == 1)
				$_SESSION["multiple_key"] .= ",". $fn->name;

			print $fn->name."</th>";
			$fieldNames .= "'" . str_replace("'", "\\'", $fn->name) . "',";
		}

		$_SESSION["primary_key"] .= ",";
		$_SESSION["unique_key"] .= ",";
		$_SESSION["multiple_key"] .= ",";

		traceMessage("keys: [".$_SESSION["primary_key"]."][".$_SESSION["unique_key"]."][".$_SESSION["multiple_key"]."]");
		print "</tr></thead><tbody>\n";

		// ------------ print data -----------
		$j = 0;
		while($r = $db->fetchRow(0, MYSQL_NUM))
		{
			$i = 0;
			print "<tr id=\"rc$j\" class=\"row\">";
			print "<td class=\"tj\">".($j+1)."</td>";

			if ($ed)
				print "<td class=\"tch\"><input name=\"ckr\" type=\"checkbox\" /></td>";

			foreach($r as $rs)
			{
				$class = ($rs === NULL) ? "tnl" : ($f[$i]->numeric == 1 ? "tr" : "tl");
				if ($_SESSION["table"]) $class .= ' edit';
				if ($f[$i]->blob == 1)
				{
					if (!is_array($_SESSION["res"][$j]))
						$_SESSION["res"][$j] = array();
					$_SESSION["res"][$j][$i] = $rs;
				}
				//if ($j%2 != 0)
				//	$class .= "2";

				$blob_length = strlen($rs);
				$data = ($rs === NULL) ? "NULL" : (($rs === "") ? "&nbsp;" :
						//($f[$i]->blob == 1 && MAX_BLOB_LENGTH_CHECK <= $f[$i]->max_length ? "#img#" : htmlspecialchars($rs)));
						($f[$i]->blob == 1 && MAX_BLOB_LENGTH_CHECK <= $blob_length ? "#img#" : htmlspecialchars($rs)));

				$extra = "";
				if ($f[$i]->blob == 1 && MAX_BLOB_LENGTH_CHECK <= $blob_length)
				{
					$btype = "";
					include("config/blobs.php");
					foreach($blobTypes as $k => $v)
					{
						if ( $v[1] && matchFileHeader($rs, $v[1]) )
						{
							traceMessage("auto detected blob type: $k");
							$btype = $k;
							break;
						}
					}
					$extra = "onclick=\"vwBlb($j, '".$f[$i]->name."', '$btype')\"";
				}

				if ($data == "#img#")
				{
					$data = $btype ?
						"<div title=\"Click to view column data\" class=\"blob\">[$blob_length Bytes]<img src=\"themes/".THEME_PATH."/images/types/".$btype.".gif\"' align=\"middle\" alt=\"\" /></div>" :
						"<div title=\"Click to view column data\" class=\"blob\">[$blob_length Bytes]<img src=\"themes/".THEME_PATH."/images/blob.gif\" align=\"middle\" alt=\"\" /></div>";
				}

				print "<td nowrap=\"nowrap\" id=\"r$j"."f$i\" class=\"$class\" $extra>$data</td>";
				$i++;
			}
			print "</tr>\n";
			$j++;
			if (MAX_RECORD_TO_DISPLAY > 0 && $j >= MAX_RECORD_TO_DISPLAY)
				break;
		}
		//if ($db->numRows() == 0)
		//	echo '<tr><td colspan="10" class="tj" style="text-align:center">No records</td></tr>';
		print "</tbody></table>";

		print "</div>";

		if (MAX_RECORD_TO_DISPLAY > 0 && $db->numRows() > MAX_RECORD_TO_DISPLAY)
			$extraMsg = "<div style='float:left;color:red;font-weight:bold'>Showing only first ".MAX_RECORD_TO_DISPLAY." records !</div>";
		else
			$extraMsg = "Query executed successfully.";
		$js = "<script type=\"text/javascript\" language=\"javascript\">\n";
		if ($_SESSION["primary_key"])
			$js .= "parent.editKey = '".$_SESSION["primary_key"]."';\n";
		else if ($_SESSION["unique_key"])
			$js .= "parent.editKey = '".$_SESSION["unique_key"]."';\n";
		else if ($_SESSION["multiple_key"])
			$js .= "parent.editKey = '".$_SESSION["multiple_key"]."';\n";
		else
			$js .= "parent.editKey = '';\n";
		$js .= "parent.editTableName = '" . $_SESSION["table"] ."';";
		//$js .=  "parent.fieldInfo = new Array(" . substr($fieldNames,0,strlen($fieldNames)-1) . ");\n" ;
		$js .=  "parent.fieldInfo = ".$fieldInfo.";";
		$tm = $db->getQueryTime();
		$js .= "parent.transferResultGrid(".$db->numRows().", '$tm', \"$extraMsg\");\n";
		$js .= "parent.addCmdHistory(\"".preg_replace("/[\n\r]/", "<br/>", htmlspecialchars($_SESSION["query"]))."\");\n";
		$js .= "</script>\n";

		print $js;
	}

	function createErrorPage()
	{
		traceMessage("createErrorPage...");

		print "<link href='cache.php?css=default' rel=stylesheet />\n";
		print "<div name='results' id='results'><table cellspacing=5 width='100%' height:100% border=0>\n";
		print "<tr><td>";
		print "<div class='warning'>The requested page is not available on the server</div>";
		print "</td></tr>";
		print "</table></div>";
		print "<script type=\"text/javascript\" language='javascript'> parent.document.updatePopupTitle('Error !'); </script>";
	}

	// numQueries = number of 'successful' executed queries
	// affectedRows = some rows maybe affected in batch processing, even if error occured
	function createErrorGrid(&$db, $query="", $numQueries=0, $affectedRows=-1)
	{
		traceMessage("createErrorGrid...");

		if ($query == "")
			$query = $_SESSION["query"];

		unset($_SESSION["res"]);
		unset($_SESSION["primary_key"]);
		unset($_SESSION["unique_key"]);
		unset($_SESSION["table"]);
		$_SESSION["res"] = array();		// result blob data
		$e = $db->getError();
		print "<div name='results' id='results'><table cellspacing=5 width='100%' border=0 class=\"msgTable\">\n";
		print "<tr><td>";
		if ($numQueries > 0)
		{
			print ($numQueries == 1 ? "1 query" : "$numQueries queries");
			print " successfully executed.<br/><br/>$affectedRows record(s) were affected.<br/><br/>";
		}
		print "</td></tr>";
		print "<tr><td>";
		print "<div class=\"warning\">Error occured while executing the query:</div><div class=\"qe\">".nl2br(htmlspecialchars($query))."</div><div>".htmlspecialchars($e)."</div>";
		print "</td></tr>";
		print "</table></div>";
		print "<script type=\"text/javascript\" language='javascript'> parent.transferResultMessage(-1, '&nbsp;', 'Error occured while executing the query');\n";
		print "parent.addCmdHistory(\"".preg_replace("/[\n\r]/", "<br/>", htmlspecialchars($_SESSION["query"]))."\");\n";
		print "</script>\n";
	}

	function createInfoGrid(&$db, $query="", $numQueries=1, $affectedRows=-1, $addHistory=true)	// batch process will send last param as false
	{
		unset($_SESSION["res"]);
		unset($_SESSION["primary_key"]);
		unset($_SESSION["unique_key"]);
		unset($_SESSION["table"]);
		$_SESSION["res"] = array();
		if ($affectedRows == -1)
			$affectedRows = $db->getAffectedRows();
		if ($query == "")
			$query = $_REQUEST["query"];
		print "<div name='results' id='results'><table cellspacing=5 width='100%' border=0 class=\"msgTable\">\n";
		print "<tr><td><div>";
		print ($numQueries == 1 ? "1 query" : "$numQueries queries");
		print " successfully executed.</div>";
		print "<div class='qi'>".nl2br(htmlspecialchars($query))."</div>";
		print "<div>$affectedRows record(s) were affected.</div>";
		print "</td></tr>";
		print "</table></div>";
		$tm = $db->getQueryTime();
		print "<script type=\"text/javascript\" language='javascript'> parent.transferResultMessage(-1, '$tm', '$affectedRows record(s) updated');\n";
		if ($addHistory)
			print "parent.addCmdHistory(\"".preg_replace("/[\n\r]/", "<br/>", htmlspecialchars($query))."\");\n";
		print "</script>\n";
	}

	function getQueryType($query)
	{
		$query = trim($query);
		$query = strtolower($query);
		if(substr($query, 0, 6) == "select" || substr($query, 0, 4) == "desc"
					|| substr($query, 0, 7) == "explain" || substr($query, 0, 4) == "show"
					|| substr($query, 0, 4) == "help" )
			$type = "result";
		else
			$type = "update";

		traceMessage("getQueryType:$query = $type");
		return $type;
	}

	function startForm($style="margin:0px")
	{
		print "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\"><html xmlns=\"http://www.w3.org/1999/xhtml\">\n";
        print "<head><title>MyWebSql</title></head><body style=\"$style\">\n";
		print "<form name='frmquery' id='frmquery' method='post' action='#' enctype='multipart/form-data' onsubmit='return false'>";
		print "<input type='hidden' name='type' value='query' />";
		print "<input type='hidden' name='query' value='' />";
		print "<input type='hidden' name='id' value='' />";
		print "<input type='hidden' name='name' value='' />";
		print "<script language='javascript' type='javascript' src='cache.php?script=common,browser'></script>\n";
		print "<script language='javascript' type='javascript'> var EXTERNAL_PATH = '".EXTERNAL_PATH."'; </script>\n";
	}

	function batchProcessQuery(&$db, $query)
	{
		$temp = tmpfile();
		fwrite($temp, $query);
		fseek($temp, 0);

		include("lib/sqlparser.php");
		$parser = new sqlParser($db);
		if (!$parser->parseFile($temp))
			createErrorGrid($db, $parser->getLastQuery(), $parser->getExecutedQueries(), $parser->getRowsAffected());	//$parser->getError();
		else
			createInfoGrid($db, $query, $parser->getExecutedQueries(), $parser->getRowsAffected(), false);	//print "All queries executed successfully";
	}
?>